@@ -1,6 +1,10 @@
 /* -----------------------------------------------------------------------------
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * cwrap.c
  *
@@ -8,7 +12,7 @@
  * the naming of local variables, calling conventions, and so forth.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_cwrap_c[] = "$Id: cwrap.c 11312 2009-06-24 17:20:17Z wsfulton $";
+char cvsroot_cwrap_c[] = "$Id: cwrap.c 12536 2011-03-14 07:22:08Z wsfulton $";
 
 #include "swig.h"
 
@@ -212,7 +216,7 @@ int Swig_cargs(Wrapper *w, ParmList *p) {
 	  SwigType_del_reference(tvalue);
 	  tycode = SwigType_type(tvalue);
 	  if (tycode != T_USER) {
-	    /* plain primitive type, we copy the the def value */
+	    /* plain primitive type, we copy the def value */
 	    String *lstr = SwigType_lstr(tvalue, defname);
 	    defvalue = NewStringf("%s = %s", lstr, qvalue);
 	    Delete(lstr);
@@ -778,16 +782,35 @@ int Swig_add_extension_code(Node *n, const String *function_name, ParmList *parm
  * Converts a C++ method node to a function accessor function.
  * ----------------------------------------------------------------------------- */
 
-int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *director_type, int is_director) {
-  String *name, *qualifier;
+int Swig_MethodToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, int flags, SwigType *director_type, int is_director) {
+  String *name;
   ParmList *parms;
   SwigType *type;
   Parm *p;
   String *self = 0;
-
-  /* If smart pointer, change self dereferencing */
+  int is_smart_pointer_overload = 0;
+  String *qualifier = Getattr(n, "qualifier");
+  
+  /* If smart pointer without const overload or mutable method, change self dereferencing */
   if (flags & CWRAP_SMART_POINTER) {
-    self = NewString("(*this)->");
+    if (flags & CWRAP_SMART_POINTER_OVERLOAD) {
+      if (qualifier && strncmp(Char(qualifier), "q(const)", 8) == 0) {
+        self = NewString("(*(this))->");
+        is_smart_pointer_overload = 1;
+      }
+      else if (Cmp(Getattr(n, "storage"), "static") == 0) {
+	String *cname = Getattr(n, "classname") ? Getattr(n, "classname") : classname;
+	String *ctname = SwigType_namestr(cname);
+        self = NewStringf("(*(%s const *)this)->", ctname);
+        is_smart_pointer_overload = 1;
+	Delete(ctname);
+      }
+      else {
+        self = NewString("(*this)->");
+      }
+    } else {
+      self = NewString("(*this)->");
+    }
   }
 
   /* If node is a member template expansion, we don't allow added code */
@@ -795,7 +818,6 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
     flags &= ~(CWRAP_EXTEND);
 
   name = Getattr(n, "name");
-  qualifier = Getattr(n, "qualifier");
   parms = CopyParmList(nonvoid_parms(Getattr(n, "parms")));
 
   type = NewString(classname);
@@ -803,7 +825,7 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
     SwigType_push(type, qualifier);
   }
   SwigType_add_pointer(type);
-  p = NewParm(type, "self");
+  p = NewParm(type, "self", n);
   Setattr(p, "self", "1");
   Setattr(p, "hidden","1");
   /*
@@ -826,7 +848,7 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
      These two lines just transfer the ownership of the 'this' pointer
      from the input to the output wrapping object.
 
-     This happens in python, but may also happens in other target
+     This happens in python, but may also happen in other target
      languages.
    */
   if (GetFlag(n, "feature:self:disown")) {
@@ -892,7 +914,7 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
     String *defaultargs = Getattr(n, "defaultargs");
     String *code = Getattr(n, "code");
     String *cname = Getattr(n, "classname") ? Getattr(n, "classname") : classname;
-    String *membername = Swig_name_member(cname, name);
+    String *membername = Swig_name_member(nspace, cname, name);
     String *mangled = Swig_name_mangle(membername);
     int is_smart_pointer = flags & CWRAP_SMART_POINTER;
 
@@ -917,10 +939,18 @@ int Swig_MethodToFunction(Node *n, String *classname, int flags, SwigType *direc
 
       if (Cmp(Getattr(n, "storage"), "static") != 0) {
 	String *pname = Swig_cparm_name(pp, i);
-        String *ctname = SwigType_namestr(cname);
-	String *fadd = NewStringf("(%s*)(%s)->operator ->()", ctname, pname);
+	String *ctname = SwigType_namestr(cname);
+	String *fadd = 0;
+	if (is_smart_pointer_overload) {
+	  String *nclassname = SwigType_namestr(classname);
+	  fadd = NewStringf("(%s const *)((%s const *)%s)->operator ->()", ctname, nclassname, pname);
+	  Delete(nclassname);
+	}
+	else {
+	  fadd = NewStringf("(%s*)(%s)->operator ->()", ctname, pname);
+	}
 	Append(func, fadd);
-        Delete(ctname);
+	Delete(ctname);
 	Delete(fadd);
 	Delete(pname);
 	pp = nextSibling(pp);
@@ -1005,16 +1035,14 @@ Node *Swig_directormap(Node *module, String *type) {
  * This function creates a C wrapper for a C constructor function. 
  * ----------------------------------------------------------------------------- */
 
-int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparison, String *director_ctor, int cplus, int flags) {
+int Swig_ConstructorToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, String *none_comparison, String *director_ctor, int cplus, int flags) {
   ParmList *parms;
   Parm *prefix_args;
   Parm *p;
   ParmList *directorparms;
   SwigType *type;
-  Node *classNode;
   int use_director;
 
-  classNode = Swig_methodclass(n);
   use_director = Swig_directorclass(n);
 
   parms = CopyParmList(nonvoid_parms(Getattr(n, "parms")));
@@ -1045,7 +1073,7 @@ int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparis
     String *cres;
     String *defaultargs = Getattr(n, "defaultargs");
     String *code = Getattr(n, "code");
-    String *membername = Swig_name_construct(classname);
+    String *membername = Swig_name_construct(nspace, classname);
     String *mangled = Swig_name_mangle(membername);
 
     /* Check if the constructor is overloaded.   If so, and it has code attached, we append an extra suffix
@@ -1149,13 +1177,13 @@ int Swig_ConstructorToFunction(Node *n, String *classname, String *none_comparis
  * This function creates a C wrapper for a destructor function.
  * ----------------------------------------------------------------------------- */
 
-int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags) {
+int Swig_DestructorToFunction(Node *n, const_String_or_char_ptr nspace, String *classname, int cplus, int flags) {
   SwigType *type;
   Parm *p;
 
   type = NewString(classname);
   SwigType_add_pointer(type);
-  p = NewParm(type, "self");
+  p = NewParm(type, "self", n);
   Setattr(p, "self", "1");
   Setattr(p, "hidden", "1");
   Setattr(p, "wrap:disown", "1");
@@ -1166,7 +1194,7 @@ int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
     String *cres;
     String *call;
     String *membername, *mangled, *code;
-    membername = Swig_name_destroy(classname);
+    membername = Swig_name_destroy(nspace, classname);
     mangled = Swig_name_mangle(membername);
     code = Getattr(n, "code");
     if (code) {
@@ -1207,7 +1235,7 @@ int Swig_DestructorToFunction(Node *n, String *classname, int cplus, int flags)
  * This function creates a C wrapper for setting a structure member.
  * ----------------------------------------------------------------------------- */
 
-int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **call) {
+int Swig_MembersetToFunction(Node *n, String *classname, int flags) {
   String *name;
   ParmList *parms;
   Parm *p;
@@ -1215,10 +1243,7 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   SwigType *ty;
   SwigType *type;
   SwigType *void_type = NewString("void");
-  String *membername;
-  String *mangled;
   String *self = 0;
-  String *sname;
 
   int varcref = flags & CWRAP_NATURAL_VAR;
 
@@ -1232,19 +1257,15 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   name = Getattr(n, "name");
   type = Getattr(n, "type");
 
-  sname = Swig_name_set(name);
-  membername = Swig_name_member(classname, sname);
-  mangled = Swig_name_mangle(membername);
-
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t, "self");
+  parms = NewParm(t, "self", n);
   Setattr(parms, "self", "1");
   Setattr(parms, "hidden","1");
   Delete(t);
 
   ty = Swig_wrapped_member_var_type(type, varcref);
-  p = NewParm(ty, name);
+  p = NewParm(ty, name, n);
   Setattr(parms, "hidden", "1");
   set_nextSibling(parms, p);
 
@@ -1255,21 +1276,32 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   Delete(p);
 
   if (flags & CWRAP_EXTEND) {
+    String *call;
     String *cres;
     String *code = Getattr(n, "code");
+
+    String *sname = Swig_name_set(0, name);
+    String *membername = Swig_name_member(0, classname, sname);
+    String *mangled = Swig_name_mangle(membername);
+
     if (code) {
       /* I don't think this ever gets run - WSF */
       Swig_add_extension_code(n, mangled, parms, void_type, code, cparse_cplusplus, "self");
     }
-    *call = Swig_cfunction_call(mangled, parms);
-    cres = NewStringf("%s;", *call);
+    call = Swig_cfunction_call(mangled, parms);
+    cres = NewStringf("%s;", call);
     Setattr(n, "wrap:action", cres);
+
     Delete(cres);
+    Delete(call);
+    Delete(mangled);
+    Delete(membername);
+    Delete(sname);
   } else {
-    String *cres;
-    *call = Swig_cmemberset_call(name, type, self, varcref);
-    cres = NewStringf("%s;", *call);
+    String *call = Swig_cmemberset_call(name, type, self, varcref);
+    String *cres = NewStringf("%s;", call);
     Setattr(n, "wrap:action", cres);
+    Delete(call);
     Delete(cres);
   }
   Setattr(n, "type", void_type);
@@ -1277,9 +1309,6 @@ int Swig_MembersetToFunction(Node *n, String *classname, int flags, String **cal
   Delete(parms);
   Delete(ty);
   Delete(void_type);
-  Delete(membername);
-  Delete(sname);
-  Delete(mangled);
   Delete(self);
   return SWIG_OK;
 }
@@ -1296,10 +1325,7 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   SwigType *t;
   SwigType *ty;
   SwigType *type;
-  String *membername;
-  String *mangled;
   String *self = 0;
-  String *gname;
 
   int varcref = flags & CWRAP_NATURAL_VAR;
 
@@ -1308,6 +1334,10 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
       Node *sn = Getattr(n, "cplus:staticbase");
       String *base = Getattr(sn, "name");
       self = NewStringf("%s::", base);
+    } else if (flags & CWRAP_SMART_POINTER_OVERLOAD) {
+      String *nclassname = SwigType_namestr(classname);
+      self = NewStringf("(*(%s const *)this)->", nclassname);
+      Delete(nclassname);
     } else {
       self = NewString("(*this)->");
     }
@@ -1319,13 +1349,9 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   name = Getattr(n, "name");
   type = Getattr(n, "type");
 
-  gname = Swig_name_get(name);
-  membername = Swig_name_member(classname, gname);
-  mangled = Swig_name_mangle(membername);
-
   t = NewString(classname);
   SwigType_add_pointer(t);
-  parms = NewParm(t, "self");
+  parms = NewParm(t, "self", n);
   Setattr(parms, "self", "1");
   Setattr(parms, "hidden","1");
   Delete(t);
@@ -1334,8 +1360,12 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   if (flags & CWRAP_EXTEND) {
     String *call;
     String *cres;
-
     String *code = Getattr(n, "code");
+
+    String *gname = Swig_name_get(0, name);
+    String *membername = Swig_name_member(0, classname, gname);
+    String *mangled = Swig_name_mangle(membername);
+
     if (code) {
       /* I don't think this ever gets run - WSF */
       Swig_add_extension_code(n, mangled, parms, ty, code, cparse_cplusplus, "self");
@@ -1343,8 +1373,12 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
     call = Swig_cfunction_call(mangled, parms);
     cres = Swig_cresult(ty, "result", call);
     Setattr(n, "wrap:action", cres);
+
     Delete(cres);
     Delete(call);
+    Delete(mangled);
+    Delete(membername);
+    Delete(gname);
   } else {
     String *call = Swig_cmemberget_call(name, type, self, varcref);
     String *cres = Swig_cresult(ty, "result", call);
@@ -1356,9 +1390,6 @@ int Swig_MembergetToFunction(Node *n, String *classname, int flags) {
   Setattr(n, "parms", parms);
   Delete(parms);
   Delete(ty);
-  Delete(membername);
-  Delete(gname);
-  Delete(mangled);
 
   return SWIG_OK;
 }
@@ -1381,10 +1412,10 @@ int Swig_VarsetToFunction(Node *n, int flags) {
   type = Getattr(n, "type");
   nname = SwigType_namestr(name);
   ty = Swig_wrapped_var_type(type, varcref);
-  parms = NewParm(ty, name);
+  parms = NewParm(ty, name, n);
 
   if (flags & CWRAP_EXTEND) {
-    String *sname = Swig_name_set(name);
+    String *sname = Swig_name_set(0, name);
     String *mangled = Swig_name_mangle(sname);
     String *call = Swig_cfunction_call(mangled, parms);
     String *cres = NewStringf("%s;", call);
@@ -1438,7 +1469,7 @@ int Swig_VargetToFunction(Node *n, int flags) {
   ty = Swig_wrapped_var_type(type, varcref);
 
   if (flags & CWRAP_EXTEND) {
-    String *sname = Swig_name_get(name);
+    String *sname = Swig_name_get(0, name);
     String *mangled = Swig_name_mangle(sname);
     call = Swig_cfunction_call(mangled, 0);
     cres = Swig_cresult(ty, "result", call);
